//  PyCOM - Distributed Component Model for Python - C++ Client Library
//  Copyright (c) 2011-2012, Dmitry "Divius" Tantsur
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following disclaimer
//    in the documentation and/or other materials provided with the
//    distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <cassert>
#include <cerrno>
#include <iostream>
#include <memory>

#include <pycom.h>

int main() {
    std::cerr << "Make sure NS is running on tcp://127.0.0.1:2012" << std::endl;
    bool ok = true;

    pycom::Value value;

    ok = false;
    try {
        pycom::Context("");
    }
    catch (std::invalid_argument) {
        ok = true;
    }
    assert(ok);

    ok = false;
    try {
        pycom::Context("foo:bar");
    }
    catch (std::invalid_argument) {
        ok = true;
    }
    assert(ok);

    pycom::Context ctx("tcp://127.0.0.1:2012");

    std::auto_ptr<pycom::Component> ns(ctx.nameserver());
    assert(ns.get() != 0);

    ns.reset(ctx.nameserver());
    assert(ns.get() != 0);
    value.clear();
    value["interface"] = "org.pycom.nameserver";
    value = ns->invoke("locate", value);
    assert(!value.isNull());

    value.clear();
    value["interface"] = "org.pycom.nameserver";
    ok = false;
    try {
        ns->invoke("locate1", value);
    }
    catch (pycom::LookupError) {
        ok = true;
    }
    assert(ok);

    value.clear();
    value["interface1"] = "org.pycom.nameserver";
    ok = false;
    try {
        ns->invoke("locate", value);
    }
    catch (pycom::BadRequest &err) {
        ok = err.code() == pycom::ERROR_BAD_REQUEST;
    }
    assert(ok);

    value.clear();
    value["interface"] = "org.pycom1.nameserver1";
    ok = false;
    try {
        ns->invoke("locate", value);
    }
    catch (pycom::RemoteError &err) {
        ok = err.code() == pycom::ERROR_SERVICE_NOT_FOUND;
    }
    assert(ok);

    std::auto_ptr<pycom::Component> remote(ctx.locate("org.pycom.nameserver"));
    value.clear();
    value["interface"] = "org.pycom.nameserver";
    value = remote->invoke("locate", value);
    assert(!value.isNull());

    ok = false;
    try {
        ctx.locate("org.pycom1.nameserver1");
    }
    catch (pycom::LookupError) {
        ok = true;
    }
    assert(ok);

    std::cerr << "Tests passed" << std::endl;
    return 0;
}
